import { BasicMouseHandler } from '@lumino/datagrid';
import { Drag } from '@lumino/dragdrop';
import { Signal } from '@lumino/signaling';
import { renderSelection } from './selection';
export class RichMouseHandler extends BasicMouseHandler {
    constructor(options) {
        super();
        this._rightClickSignal = new Signal(this);
        this._grid = options.grid;
        this._cursor = null;
    }
    get rightClickSignal() {
        return this._rightClickSignal;
    }
    cursorForHandle(region) {
        const cursorMap = {
            top: 'ns-resize',
            left: 'ew-resize',
            right: 'ew-resize',
            bottom: 'ns-resize',
            none: this.cursorByRegion()
        };
        this._cursor = cursorMap[region];
        return this._cursor;
    }
    cursorByRegion() {
        const hit = this._grid.hitTest(this._event.clientX, this._event.clientY);
        switch (hit.region) {
            case 'row-header': {
                return 'grab';
            }
            case 'column-header': {
                return 'grab';
            }
            default: {
                return 'default';
            }
        }
    }
    onMouseHover(grid, event) {
        this._event = event;
        super.onMouseHover(grid, event);
    }
    /**
     * Handle the mouse down event for the data grid.
     *
     * @param grid - The data grid of interest.
     *
     * @param event - The mouse down event of interest.
     */
    onMouseDown(grid, event) {
        this._event = event;
        super.onMouseDown(grid, event);
        if (this._cursor === 'grab') {
            this._cursor = 'grabbing';
            this.handleGrabbing();
        }
        return;
    }
    release() {
        if (this._moveData) {
            this._moveData.override.dispose();
            this._moveData = null;
        }
        super.release();
    }
    handleGrabbing() {
        const hit = this._grid.hitTest(this._event.clientX, this._event.clientY);
        const { region, row, column } = hit;
        // get row/column geometry
        // the real parameters for the shadow rectangle
        let [r1, r2, c1, c2] = this.getShadowRegion(region, row, column);
        // get the left and top offsets of the grid viewport
        const { left, top } = this._grid.viewport.node.getBoundingClientRect();
        // get the bounds for dragging
        let lowerBound;
        let upperBound;
        let rightBound;
        let leftBound;
        if (region === 'column-header') {
            lowerBound = upperBound = r1;
            leftBound = left + this._grid.headerWidth;
            rightBound =
                left +
                    Math.min(this._grid.pageWidth - (c2 - c1), this._grid.headerWidth + this._grid.bodyWidth - (c2 - c1));
        }
        else if (region === 'row-header') {
            lowerBound =
                top +
                    Math.min(this._grid.pageHeight - (r2 - r1), this._grid.bodyHeight + this._grid.headerHeight - (r2 - r1));
            upperBound = top + this._grid.headerHeight;
            leftBound = rightBound = c1;
        }
        const boundingRegion = {
            upperBound: upperBound,
            lowerBound: lowerBound,
            leftBound: leftBound,
            rightBound: rightBound
        };
        renderSelection(r1, r2, c1, c2, this._event.clientX, this._event.clientY, boundingRegion, 'shadow');
        // initiate the dark line
        [r1, r2, c1, c2] = this.getLineRegionFromShadowRegion(r1, r2, c1, c2, this._event, region);
        this._moveLine = renderSelection(r1, r2, c1, c2, this._event.clientX, this._event.clientY);
        // Create the move data
        if (region === 'void') {
            return;
        }
        const type = 'move';
        // Override the document cursor.
        const override = Drag.overrideCursor('grabbing');
        this._moveData = {
            type,
            region,
            row,
            column,
            override,
            localX: -1,
            localY: -1,
            timeout: -1
        };
        return;
    }
    getShadowRegion(region, row, column) {
        let r1;
        let r2;
        let c1;
        let c2;
        // get the left and top offsets of the grid viewport
        const { left, top } = this._grid.viewport.node.getBoundingClientRect();
        if (region === 'column-header') {
            r1 = top + this._grid.headerHeight;
            r2 =
                top +
                    Math.min(this._grid.pageHeight + this._grid.headerHeight, this._grid.bodyHeight + this._grid.headerHeight);
            c1 =
                left + this._grid.headerWidth + this._grid.columnOffset('body', column);
            c2 = c1 + this._grid.columnSize('body', column);
        }
        else if (region === 'row-header') {
            r1 = top + this._grid.headerHeight + this._grid.rowOffset('body', row);
            r2 = r1 + this._grid.rowSize('body', row);
            c1 = left + this._grid.headerWidth;
            c2 =
                left +
                    Math.min(this._grid.pageWidth + this._grid.headerWidth, this._grid.headerWidth + this._grid.bodyWidth);
        }
        return [r1, r2, c1, c2];
    }
    /**
     * Handle the mouse move event for the data grid.
     *
     * @param grid - The data grid of interest.
     *
     * @param event - The mouse move event of interest.
     */
    onMouseMove(grid, event) {
        // Fetch the press data.
        if (this._moveData) {
            this.updateLinePos(grid, event);
        }
        else {
            super.onMouseMove(grid, event);
        }
        return;
    }
    /**
     * Moves the line based on the position of the cursor and shadow
     * @param grid
     * @param event
     */
    updateLinePos(grid, event) {
        // reduce the sensitivity slightly
        if (Math.abs(event.movementX) < 0.5 && Math.abs(event.movementY) < 0.5) {
            return;
        }
        const hit = grid.hitTest(event.clientX, event.clientY);
        const { region, row, column } = hit;
        let [r1, r2, c1, c2] = this.getShadowRegion(region, row, column);
        [r1, r2, c1, c2] = this.getLineRegionFromShadowRegion(r1, r2, c1, c2, event, region);
        this._moveLine.manualPositionUpdate(c1, r1);
    }
    getLineRegionFromShadowRegion(r1, r2, c1, c2, event, region) {
        if (region === 'column-header') {
            event.movementX > 0 ? (c1 = c2 + 1) : (c2 = c1 + 1);
        }
        else if (region === 'row-header') {
            event.movementY > 0 ? (r1 = r2 + 1) : (r2 = r1 + 1);
        }
        return [r1, r2, c1, c2];
    }
    /**
     *
     * @param grid
     * @param event
     */
    onMouseUp(grid, event) {
        if (this._moveData) {
            let { vx, vy } = grid.mapToVirtual(event.clientX, event.clientY);
            // Clamp the coordinates to the limits.
            vx = Math.max(0, Math.min(vx, grid.bodyWidth - 1));
            vy = Math.max(0, Math.min(vy, grid.bodyHeight - 1));
            const model = grid.dataModel;
            if (this._moveData.region === 'column-header') {
                const startColumn = this._moveData.column;
                const endColumn = grid.columnAt('body', vx);
                model.moveColumn(startColumn, endColumn);
            }
            else if (this._moveData.region === 'row-header') {
                const startRow = this._moveData.row;
                const endRow = grid.rowAt('body', vy);
                model.moveRow(startRow, endRow);
            }
        }
        this.release();
        return;
    }
    /**
     * Handle the context menu event for the data grid.
     *
     * @param grid - The data grid of interest.
     *
     * @param event - The context menu event of interest.
     */
    onContextMenu(grid, event) {
        const { clientX, clientY } = event;
        const hit = grid.hitTest(clientX, clientY);
        const { row, column } = hit;
        this._rightClickSignal.emit([row, column]);
    }
}
//# sourceMappingURL=handler.js.map