import { MutableDataModel, DataModel } from '@lumino/datagrid';
import { DSVModel } from 'tde-csvviewer';
import { Signal } from '@lumino/signaling';
import { Litestore } from './litestore';
export declare class EditableDSVModel extends MutableDataModel {
    private _clipBoardArr;
    constructor(options: DSVModel.IOptions);
    get dsvModel(): DSVModel;
    get cancelEditingSignal(): Signal<this, null>;
    get onChangedSignal(): Signal<this, string>;
    get colHeaderLength(): number;
    get litestore(): Litestore;
    private _silenceDsvModel;
    private _passMessage;
    rowCount(region?: DataModel.RowRegion): number;
    columnCount(region?: DataModel.ColumnRegion): number;
    metadata(region: DataModel.CellRegion, row: number, column: number): DataModel.Metadata;
    data(region: DataModel.CellRegion, row: number, column: number): any;
    setData(region: DataModel.CellRegion, row: number, column: number, value: any, useLitestore?: boolean): boolean;
    /**
     * Adds a row to the body of the model
     * @param row The index of the row to be inserted (0-indexed)
     */
    addRow(row: number): void;
    /**
     * Adds a column to the body of the model
     * @param column The index of the column to be inserted (0-indexed)
     */
    addColumn(column: number): void;
    /**
     * Removes a row from the body of the model
     * @param row The index of the row removed (0-indexed)
     */
    removeRow(row: number): void;
    /**
     * Removes a column from the body of the model
     * @param column The index of the column removed (0-indexed)
     */
    removeColumn(column: number): void;
    /**
     * Copies the current selection and potential removes it (cut)
     * @param selection The current selction
     * @param mode 'cut-cells' or 'copy-cells'
     */
    cutAndCopy(selection: ICellSelection, mode?: 'cut-cells' | 'copy-cells'): void;
    /**
     * Pastes the current data from the clipboard
     * @param startCoord The start coordinates of the paste
     * @param data The data being pasted
     */
    paste(startCoord: ICoordinates, data?: string | null): void;
    /**
     * Utilizes the litestore to undo the last change
     * @param change The arguments of the last change
     */
    undo(change: DataModel.ChangedArgs): void;
    /**
     * Utilizes the litestore to redo the last undo
     * @param change The arguments of the change to be redone
     */
    redo(change: DataModel.ChangedArgs, modelData: string): void;
    /**
     * Swaps the data between two rows
     * @param startRow The index of the row to be moved (0-indexed)
     * @param endRow The index of the other row to be moved (0-indexed)
     */
    moveRow(startRow: number, endRow: number): void;
    /**
     * Swaps the data between two columns
     * @param startColumn The index of the column to be moved (0-indexed)
     * @param endColumn The index of the other column to be moved (0-indexed)
     */
    moveColumn(startColumn: number, endColumn: number): void;
    /**
     * Handles all singal emitting and model parsing after the raw data is manipulated
     * @param change The current change to be emitted to the Datagrid
     */
    handleEmits(change: DataModel.ChangedArgs): void;
    sliceOut(model: DSVModel, cellLoc: ICoordinates, keepingCell?: boolean, keepingValue?: boolean): string;
    insertAt(value: any, model: DSVModel, cellLoc: ICoordinates): void;
    /**
     * Returns a blank row with the correct numbers of columns and correct delimiters
     * @param model The DSV model being used
     * @param row The index of the row being inserted (determines whether to add a row delimiter or not)
     */
    blankRow(model: DSVModel, row: number): string;
    /**
     * @param coords: the coordinates of the cell.
     */
    firstIndex(coords: ICoordinates): number;
    lastIndex(coords: ICoordinates): number;
    rowEnd(row: number): number;
    isTrimOperation(coords: ICoordinates): boolean;
    isExtensionOperation(coords: ICoordinates): boolean;
    getPreviousCell(coords: ICoordinates): ICoordinates;
    getNextCell(coords: ICoordinates): ICoordinates;
    /**
     * Creates a new transaction containing the raw data, header, and changeArgs
     * @param change The change args for the Datagrid (may be null)
     */
    updateLitestore(change?: DataModel.ChangedArgs): void;
    private _dsvModel;
    private _onChangeSignal;
    private _transmitting;
    private _cancelEditingSignal;
    private _litestore;
}
interface ICoordinates {
    row: number;
    column?: number;
}
export interface ICellSelection {
    startRow: number;
    startColumn: number;
    endColumn: number;
    endRow: number;
}
export declare const SCHEMA_ID = "datamodel";
export declare const RECORD_ID = "datamodel";
export declare const DATAMODEL_SCHEMA: {
    id: string;
    fields: {
        modelData: import("@lumino/datastore").RegisterField<string>;
        modelHeader: import("@lumino/datastore").RegisterField<string[]>;
        change: import("@lumino/datastore").RegisterField<DataModel.ChangedArgs>;
    };
};
export {};
